package com.yunzhi.niucai.alipay;

import com.alipay.api.AlipayApiException;
import com.alipay.api.AlipayClient;
import com.alipay.api.DefaultAlipayClient;
import com.alipay.api.request.AlipaySystemOauthTokenRequest;
import com.alipay.api.response.AlipaySystemOauthTokenResponse;
import com.yunzhi.niucai.entity.TaAlipay;
import com.yunzhi.niucai.service.ITaAlipayService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

@Slf4j
@Component
public class AlipayFactory implements InitializingBean {
    private AlipayClient alipayClient;

    @Resource(name = "taShopAppServiceImpl")
    private ITokenResolve tokenResolve;

    @Autowired
    private Config config;

    @Autowired
    ITaAlipayService iTaAlipayService;

    public AliPayService getService(Integer shopId) {
        return new AliPayService(shopId, alipayClient, tokenResolve, config);
    }

    private AlipayClient getAlipayClient(Config config) {
        return new DefaultAlipayClient(config.getApiURL(),
                config.getAppid(),
                config.getPrivateKey(),
                config.getFormat(),
                config.getCharset(),
                config.getPublicKey(),
                config.getSignType());
    }

    public String getAppID() {
        return config.getAppid();
    }

    /**
     * 获取授权用户ID
     * @param authCode
     * @return
     * @throws Exception
     */
    public String getUserId(String authCode) throws Exception {
        AlipaySystemOauthTokenRequest oauthTokenRequest = new AlipaySystemOauthTokenRequest();
        oauthTokenRequest.setCode(authCode);
        oauthTokenRequest.setGrantType("authorization_code");
        try {
            AlipaySystemOauthTokenResponse oauthTokenResponse = alipayClient.execute(oauthTokenRequest);
            return oauthTokenResponse.getUserId();
        } catch (AlipayApiException e) {
            log.error("授权支付宝用户信息失败: {}", e.getErrMsg());
            e.printStackTrace();
            throw e;
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        // 写死, 取第一个
        TaAlipay taAlipay = iTaAlipayService.getById(1);
        this.config.setAppid(taAlipay.getAppid())
                .setPrivateKey(taAlipay.getMerchantPrivateKey())
                .setPublicKey(taAlipay.getAlipayPublicKey());

        //
        this.alipayClient = getAlipayClient(this.config);
    }
}
