package com.yunzhi.niucai.controller;

import com.alibaba.fastjson.JSONObject;
import com.alipay.api.AlipayApiException;
import com.yunzhi.niucai.alipay.AlipayFactory;
import com.yunzhi.niucai.alipay.AliPayService;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.entity.TaShopkeeper;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.service.ITaShopService;
import com.yunzhi.niucai.service.ITaShopkeeperService;
import com.yunzhi.niucai.vo.AlipayAuthUser;
import com.yunzhi.niucai.vo.SportDicts;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
    *  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-27
 */

@Slf4j
@Api(tags = "")
@RestController
@RequestMapping("/")
public class TaAlipayController extends BaseController {

    @Autowired
    private AlipayFactory alipayFactory;

    @Autowired
    private ITaShopService iTaShopService;

    @Autowired
    private ITaShopkeeperService iTaShopkeeperService;

    @GetMapping("/shop/alipay/appid")
    @ApiOperation(value="获取平台支付宝应用AppID", notes = "获取平台支付宝应用AppID", httpMethod = "GET", response = SportDicts.class)
    public ResponseBean getPlatAppID() {
        return ResponseBean.success(alipayFactory.getAppID());
    }

    @PostMapping("/shop/alipay/auth/user")
    @ApiOperation(value="绑定支付宝用户", notes = "绑定支付宝用户", httpMethod = "GET", response = SportDicts.class)
    public ResponseBean authUser(@RequestBody AlipayAuthUser alipayAuthUser,
                                 HttpServletRequest request) throws Exception {
        if (null == alipayAuthUser || StringUtils.isEmpty(alipayAuthUser.getAuthCode())) {
            return ResponseBean.error("未设置授权码", ResponseBean.ERROR_MISSING_PARAMS);
        }

        TaShopkeeper taShopkeeper = getShopKeeper(request);
        if (null == taShopkeeper || StatusEnum.DELETED.getCode().equals(taShopkeeper.getStatus())) {
            return ResponseBean.error("账户登录状态异常, 请重新登录", ResponseBean.ERROR_UNAVAILABLE);
        }

        String aliUserId = alipayFactory.getUserId(alipayAuthUser.getAuthCode());
        boolean b = iTaShopkeeperService.bindThirdPartId(taShopkeeper, aliUserId, "ali");

        if (b) {
            taShopkeeper.setIsValidAli(true);
            iTaShopkeeperService.updateById(taShopkeeper);
        }

        return b ? ResponseBean.success("success") : ResponseBean.error("授权用户失败", ResponseBean.ERROR_UNAVAILABLE);
    }

    /**
     * 应用授权成功回调
     * @param appId
     * @param appAuthCode
     * @throws AlipayApiException
     */
    @GetMapping("/app/callback/alipay/auth")
    public void aliPayAuthNotice(@RequestParam("app_id") String appId,
                                 @RequestParam("app_auth_code") String appAuthCode) throws AlipayApiException {
        // 虚拟一个服务, 主要是为了调用其中方法
        AliPayService payService = alipayFactory.getService(-1);
        payService.persistToken(appAuthCode);
    }

    @GetMapping("/app/callback/alipay")
    public void aliPayCallback(@RequestParam("app_id") String appId,
                               @RequestParam("msg_app_id") String shopAppId,
                               @RequestParam("msg_method") String msgMethod,
                               @RequestParam("biz_content") String content,
                               HttpServletRequest request) {

        // 转账结果提醒
        if ("alipay.fund.trans.order.changed".equals(msgMethod)) {
            try {
                JSONObject transRes = JSONObject.parseObject(content);
                String actionType = transRes.getString("action_type");
                if (StringUtils.isEmpty(actionType)) {
                    log.error("解析转账结果提醒失败: 内容【{}】", content);
                    return;
                }

                // 支付宝有 CREATE-创建, FINISH- 订单处理已完结, CLOSE-超时关闭
                // 三种提醒
                if (!"CREATE".equals(actionType)) {
                    String outBizNO = transRes.getString("out_biz_no");
                    String status = transRes.getString("status");
                    String payDate = transRes.getString("pay_date");

                }

            } catch (Exception e) {
                log.error("解析转账结果提醒失败: {}", e.getMessage());
                e.printStackTrace();
            }
        }
    }
}
