package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.common.SMSCaptcha;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.entity.TaShopkeeper;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.service.ITaShopService;
import com.yunzhi.niucai.vo.CustIdentityInfo;
import com.yunzhi.niucai.vo.SignUpParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITaCustomerService;
import com.yunzhi.niucai.entity.TaCustomer;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
    * 客户 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-08-28
 */

@Api(tags = "用户管理")
@RestController
@RequestMapping("/")
public class TaCustomerController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaCustomerController.class);

    @Autowired
    SMSCaptcha smsCaptcha;

    @Autowired
    public ITaCustomerService iTaCustomerService;

    @Autowired
    private ITaShopService iTaShopService;

    @PostMapping("/app/signup")
    @ApiOperation(value="APP 用户注册", notes = "APP 用户注册", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean signUp(@ApiParam("注册参数") @RequestBody SignUpParam signUpParam) throws Exception {
        if (null == signUpParam) {
            return ResponseBean.error("请填写注册信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getPhone())) {
            return ResponseBean.error("请填写手机号码", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getCaptcha())) {
            return ResponseBean.error("请填写手机验证码", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getPassword())) {
            return ResponseBean.error("请填写登录密码", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (!smsCaptcha.validate(signUpParam.getPhone(), signUpParam.getCaptcha())) {
            return ResponseBean.error("验证码无效或者已过期", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        iTaCustomerService.signUp(signUpParam);

        return ResponseBean.success("success");
    }

    @PutMapping("/app/bind/identity")
    @ApiOperation(value="绑定身份证号", notes = "绑定身份证号", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean bindIDCardNo(@ApiParam("绑定信息") @RequestBody CustIdentityInfo custIdentityInfo,
                                        HttpServletRequest request
                                     ) throws Exception {
        TaCustomer taCustomer = getCustomer(request);
        if (null == taCustomer || StatusEnum.DELETED.getCode().equals(taCustomer.getStatus())) {
            return ResponseBean.error("未登录或者已过期, 请重新登录", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (null == custIdentityInfo
                || StringUtils.isEmpty(custIdentityInfo.getIdCard())
                || StringUtils.isEmpty(custIdentityInfo.getRealName())) {
            return ResponseBean.error("用户真实姓名或者身份证号不能为空", ResponseBean.ERROR_MISSING_PARAMS);
        }

        taCustomer.setRealName(custIdentityInfo.getRealName());
        taCustomer.setIdCard(custIdentityInfo.getIdCard());

        iTaCustomerService.updateIdCard(taCustomer);
        return ResponseBean.success("success");
    }


    /**
     * 获取店铺客户
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/shop/customer",method= RequestMethod.GET)
    @ApiOperation(value="获取店铺客户", notes = "all 是所有客户, order 仅下过单的客户, recommend 被当前店铺推荐的客户", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taCustomerList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                       @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                       @ApiParam(value = "客户来源", allowableValues = "all,order,recommend") @RequestParam(value ="from",defaultValue = "all") String from,
                                       HttpServletRequest request
                                       ) throws Exception{
        TaShopkeeper taShopkeeper = getShopKeeper(request);
        TaShop taShop = iTaShopService.getProfileBy(taShopkeeper);

        IPage<TaCustomer> pg = new Page<>(pageNum, pageSize);
        IPage<TaCustomer> result = iTaCustomerService.listCustomersOf(pg, taShop, from);
        return ResponseBean.success(result);
    }
//
//    /**
//     * 保存对象
//     * @param taCustomer 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taCustomer",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taCustomerAdd(@ApiParam("保存内容") @RequestBody TaCustomer taCustomer) throws Exception{
//
//        if (iTaCustomerService.save(taCustomer)){
//            return ResponseBean.success(taCustomer);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taCustomer/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taCustomerDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaCustomerService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
    /**
     * 修改对象
     * @param id  实体ID
     * @param taCustomer 实体对象
     * @return
     */
    @RequestMapping(value="/app/customer/{id}",method= RequestMethod.PUT)
    @ApiOperation(value="更新客户信息", notes = "更新客户信息", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean taCustomerUpdate(@ApiParam("当前客户ID") @PathVariable Integer id,
                                         @ApiParam("更新内容") @RequestBody TaCustomer taCustomer,
                                         HttpServletRequest request) throws Exception{
        TaCustomer customer = getCustomer(request);
        if (null == customer || StatusEnum.DELETED.getCode().equals(customer.getStatus())) {
            return ResponseBean.error("校验登录状态出错, 请重新登录", ResponseBean.ERROR_UNAVAILABLE);
        }

        if (!customer.getCustomerId().equals(id)) {
            return ResponseBean.error("不支持更新非当前人员", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        customer.setAvatar(taCustomer.getAvatar());
        customer.setPhone(taCustomer.getPhone());
        customer.setName(taCustomer.getName());
//        customer.setRealName(taCustomer.getRealName());
        customer.setAlipayUser(taCustomer.getAlipayUser());

        if (iTaCustomerService.updateById(customer)){
            customer = iTaCustomerService.getById(id);
            customer.setCashPass(null);
            customer.setLoginPass(null);
            return ResponseBean.success(customer);
        }else {
            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }

    /**
     * 根据id查询对象
     * @param id  实体ID
     */
    @RequestMapping(value="/app/customer/{id}",method= RequestMethod.GET)
    @ApiOperation(value="根据id查询用户", notes = "用户信息: 如果 id 设置为 current , 则代表查询当前用户", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taCustomerGet(@ApiParam("用户 ID") @PathVariable String id,
                                      HttpServletRequest request) throws Exception{
        TaCustomer customer = null;
        if ("current".equals(id)) {
            customer = getCustomer(request);
        } else {
            customer = iTaCustomerService.getById(id);
        }

        if (null == customer || StatusEnum.DELETED.getCode().equals(customer.getStatus())) {
            return ResponseBean.error("未查询到人员信息", ResponseBean.ERROR_UNAVAILABLE);
        }

        customer.setCashPass(null);
        customer.setLoginPass(null);
        return ResponseBean.success(customer);
    }
}
