package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.CommConstants;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.entity.TaShopkeeper;
import com.yunzhi.niucai.service.ITaShopService;
import com.yunzhi.niucai.vo.DrawOrderParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITaOrderService;
import com.yunzhi.niucai.entity.TaOrder;

import javax.servlet.http.HttpServletRequest;

/**
 * <p>
    * 订单主表  前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-09
 */

@Api(tags = "订单主表 ")
@RestController
@RequestMapping("/")
public class TaOrderController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaOrderController.class);

    @Autowired
    public ITaOrderService iTaOrderService;

    @Autowired
    public ITaShopService iTaShopService;

    /**
     * 商铺订单列表
     * @param pageNum
     * @param pageSize
     * @return
     */
    @RequestMapping(value="/shop/order",method= RequestMethod.GET)
    @ApiOperation(value="商铺订单列表", notes = "商铺订单列表", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taOrderList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
                                    @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize,
                                    @ApiParam("彩种类型, 不传就是所有") @RequestParam(value ="lotteryId",defaultValue = "") String lotteryId,
                                    HttpServletRequest request
                                    ) throws Exception{
        TaShopkeeper taShopkeeper = getShopKeeper(request);
        TaShop taShop = iTaShopService.getProfileBy(taShopkeeper);

        IPage<TaOrder> pg = new Page<>(pageNum, pageSize);
        IPage<TaOrder> result = iTaOrderService.getShopOrders(pg, taShop, lotteryId);
        return ResponseBean.success(result);
    }
//
//    /**
//     * 保存对象
//     * @param taOrder 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taOrder",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taOrderAdd(@ApiParam("保存内容") @RequestBody TaOrder taOrder) throws Exception{
//
//        if (iTaOrderService.save(taOrder)){
//            return ResponseBean.success(taOrder);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taOrder/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taOrderDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaOrderService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taOrder 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taOrder/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taOrderUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaOrder taOrder) throws Exception{
//
//        if (iTaOrderService.updateById(taOrder)){
//            return ResponseBean.success(iTaOrderService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }

    /**
     * 查询订单详情
     * @param orderId  实体ID
     */
    @RequestMapping(value="/shop/order/{orderId}",method= RequestMethod.GET)
    @ApiOperation(value="查询订单详情", notes = "查询订单详情", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean getShopOrder(@ApiParam("订单ID") @PathVariable String orderId,
                                   HttpServletRequest request) throws Exception{
        TaShopkeeper shopkeeper = getShopKeeper(request);
        TaShop taShop = iTaShopService.getProfileBy(shopkeeper);

        TaOrder taOrder = iTaOrderService.getShopOrderDetail(taShop, orderId);

        return ResponseBean.success(taOrder);
    }

    @PutMapping("/shop/draw/order")
    @ApiOperation(value="商铺出票", notes = "商铺出票", httpMethod = "PUT", response = ResponseBean.class)
    public ResponseBean drawOrder(@ApiParam("出票参数") @RequestBody DrawOrderParam drawOrderParam,
                                  HttpServletRequest request) throws Exception {
        if (null == drawOrderParam || StringUtils.isEmpty(drawOrderParam.getOrderId())) {
            return ResponseBean.error("出票参数不能为空", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaShopkeeper shopkeeper = getShopKeeper(request);
        TaShop taShop = iTaShopService.getProfileBy(shopkeeper);
        if (null == taShop) {
            return ResponseBean.error("登录状态异常, 请重新登录", ResponseBean.ERROR_UNAVAILABLE);
        }

        TaOrder taOrder = iTaOrderService.getById(drawOrderParam.getOrderId());
        if (!taShop.getShopId().equals(taOrder.getShopId())) {
            return ResponseBean.error("校验订单失败, 未找到有效订单", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        boolean result = iTaOrderService.drawOrder(taOrder, drawOrderParam);
        if (result) {
            return ResponseBean.success("success");
        } else {
            return ResponseBean.error("出票失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
        }
    }


}
