package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.entity.TaShopkeeper;
import com.yunzhi.niucai.entity.TdLottery;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.service.ITaShopService;
import com.yunzhi.niucai.service.ITdLotteryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import com.yunzhi.niucai.service.ITaShopLotteryService;
import com.yunzhi.niucai.entity.TaShopLottery;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

/**
 * <p>
    * 店铺彩种设置 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-10
 */

@Api(tags = "店铺彩种设置")
@RestController
@RequestMapping("/")
public class TaShopLotteryController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaShopLotteryController.class);

    @Autowired
    public ITaShopLotteryService iTaShopLotteryService;

    @Autowired
    private ITaShopService iTaShopService;

    @Autowired
    private ITdLotteryService iTdLotteryService;


    /**
     * 获取店铺开通彩种
     * @return
     */
    @RequestMapping(value="/shop/lottery",method= RequestMethod.GET)
    @ApiOperation(value="获取店铺开通彩种", notes = "获取店铺开通彩种", httpMethod = "GET", response = ResponseBean.class)
    public ResponseBean taShopLotteryList(@ApiParam("是否包含未开通彩种") @RequestParam(value ="all",defaultValue = "false") Boolean all,
                                          HttpServletRequest request) throws Exception{

        TaShopkeeper shopkeeper = getShopKeeper(request);
        if (null == shopkeeper && !StatusEnum.DELETED.getCode().equals(shopkeeper.getStatus())) {
            return ResponseBean.error("校验店铺信息失败, 请重新登录", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TaShop taShop = iTaShopService.getProfileBy(shopkeeper);
        if (null == taShop) {
            return ResponseBean.error("校验店铺信息失败, 请重新登录", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        List<TdLottery> result = iTdLotteryService.getDictBy(taShop, all);
        return ResponseBean.success(result);
    }

    /**
     * 店铺彩种设置
     * @param taShopLottery 实体对象
     * @return
     */
    @RequestMapping(value="/shop/{shopId}/lottery/{lotteryId}",method= RequestMethod.POST)
    @ApiOperation(value="店铺彩种设置", notes = "店铺彩种设置", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean taShopLotteryAdd(@ApiParam("店铺ID") @PathVariable Integer shopId,
                                         @ApiParam("彩种ID") @PathVariable String lotteryId,
                                         @ApiParam("彩种设置内容") @RequestBody TaShopLottery taShopLottery,
                                         HttpServletRequest request) throws Exception{
        TaShopkeeper shopkeeper = getShopKeeper(request);
        if (null == shopkeeper) {
            return ResponseBean.error("校验人员信息出错, 请重新登录", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }
        TaShop taShop = iTaShopService.getProfileBy(shopkeeper);
        if (null == taShop) {
            return ResponseBean.error("校验人员信息出错, 请重新登录", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (!taShop.getShopId().equals(shopId)) {
            return ResponseBean.error("店铺店主非当前人员", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        TdLottery tdLottery = iTdLotteryService.getById(lotteryId);
        if (null == tdLottery || StatusEnum.DELETED.getCode().equals(tdLottery.getStatus())) {
            return ResponseBean.error("设置彩种不存在", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        taShopLottery.setShopId(shopId);
        taShopLottery.setLotteryId(lotteryId);
        taShopLottery.setStatus(StatusEnum.NORMAL.getCode());

        // 校验是否存在
        TaShopLottery shopLottery = iTaShopLotteryService.getSettingBy(shopId, lotteryId);
        if (null == shopLottery) {
            iTaShopLotteryService.save(taShopLottery);
        } else {
            taShopLottery.setSettingId(shopLottery.getSettingId());
            iTaShopLotteryService.updateById(taShopLottery);
        }

        return ResponseBean.success(taShopLottery);
    }

//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taShopLottery/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taShopLotteryDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaShopLotteryService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taShopLottery 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taShopLottery/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taShopLotteryUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaShopLottery taShopLottery) throws Exception{
//
//        if (iTaShopLotteryService.updateById(taShopLottery)){
//            return ResponseBean.success(iTaShopLotteryService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taShopLottery/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taShopLotteryGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iTaShopLotteryService.getById(id));
//    }
}
