package com.yunzhi.niucai.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.BaseController;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.common.SMSCaptcha;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.vo.ShopKeeperSignUp;
import com.yunzhi.niucai.vo.SignUpParam;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import com.yunzhi.niucai.service.ITaShopkeeperService;
import com.yunzhi.niucai.entity.TaShopkeeper;

/**
 * <p>
    * 店主 前端控制器
    * </p>
 *
 * @author yansen
 * @since 2020-09-28
 */

@Api(tags = "店主")
@RestController
@RequestMapping("/")
public class TaShopkeeperController extends BaseController {

    private final Logger logger = LoggerFactory.getLogger(TaShopkeeperController.class);

    @Autowired
    public ITaShopkeeperService iTaShopkeeperService;

    @Autowired
    private SMSCaptcha smsCaptcha;

    @PostMapping("/shop/signup")
    @ApiOperation(value="店铺注册", notes = "店铺注册", httpMethod = "POST", response = ResponseBean.class)
    public ResponseBean signUp(@ApiParam("注册参数") @RequestBody ShopKeeperSignUp signUpParam) throws Exception {
        if (null == signUpParam) {
            return ResponseBean.error("请填写注册信息", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getPhone())) {
            return ResponseBean.error("请填写手机号码", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getCaptcha())) {
            return ResponseBean.error("请填写手机验证码", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getPassword())) {
            return ResponseBean.error("请填写登录密码", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (!smsCaptcha.validate(signUpParam.getPhone(), signUpParam.getCaptcha())) {
            return ResponseBean.error("验证码无效或者已过期", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getShopName())) {
            return ResponseBean.error("请填写店铺名称", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        if (StringUtils.isEmpty(signUpParam.getLng()) || StringUtils.isEmpty(signUpParam.getLat())) {
            return ResponseBean.error("请设置店铺经纬度", ResponseBean.ERROR_ILLEGAL_PARAMS);
        }

        iTaShopkeeperService.signUp(signUpParam);

        return ResponseBean.success("success");
    }

//
//    /**
//     * 分页查询列表
//     * @param pageNum
//     * @param pageSize
//     * @return
//     */
//    @RequestMapping(value="/taShopkeeper",method= RequestMethod.GET)
//    @ApiOperation(value="列表", notes = "列表", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taShopkeeperList(@ApiParam("页码") @RequestParam(value ="pageNum",defaultValue = "1") Integer pageNum,
//									 @ApiParam("单页数据量") @RequestParam(value ="pageSize",defaultValue = "10") Integer pageSize) throws Exception{
//
//		    IPage<TaShopkeeper> pg = new Page<>(pageNum, pageSize);
//            QueryWrapper<TaShopkeeper> queryWrapper = new QueryWrapper<>();
//            queryWrapper.orderByDesc("create_date");
//
//            IPage<TaShopkeeper> result = iTaShopkeeperService.page(pg, queryWrapper);
//            return ResponseBean.success(result);
//    }
//
//    /**
//     * 保存对象
//     * @param taShopkeeper 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taShopkeeper",method= RequestMethod.POST)
//    @ApiOperation(value="保存", notes = "保存", httpMethod = "POST", response = ResponseBean.class)
//    public ResponseBean taShopkeeperAdd(@ApiParam("保存内容") @RequestBody TaShopkeeper taShopkeeper) throws Exception{
//
//        if (iTaShopkeeperService.save(taShopkeeper)){
//            return ResponseBean.success(taShopkeeper);
//        }else {
//            return ResponseBean.error("保存失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id删除对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taShopkeeper/{id}", method= RequestMethod.DELETE)
//    @ApiOperation(value="删除", notes = "删除", httpMethod = "DELETE", response = ResponseBean.class)
//    public ResponseBean taShopkeeperDelete(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        if(iTaShopkeeperService.removeById(id)){
//            return ResponseBean.success("success");
//        }else {
//            return ResponseBean.error("删除失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 修改对象
//     * @param id  实体ID
//     * @param taShopkeeper 实体对象
//     * @return
//     */
//    @RequestMapping(value="/taShopkeeper/{id}",method= RequestMethod.PUT)
//    @ApiOperation(value="更新", notes = "更新", httpMethod = "PUT", response = ResponseBean.class)
//    public ResponseBean taShopkeeperUpdate(@ApiParam("对象ID") @PathVariable Integer id,
//                                        @ApiParam("更新内容") @RequestBody TaShopkeeper taShopkeeper) throws Exception{
//
//        if (iTaShopkeeperService.updateById(taShopkeeper)){
//            return ResponseBean.success(iTaShopkeeperService.getById(id));
//        }else {
//            return ResponseBean.error("修改失败, 请重试", ResponseBean.ERROR_UNAVAILABLE);
//        }
//    }
//
//    /**
//     * 根据id查询对象
//     * @param id  实体ID
//     */
//    @RequestMapping(value="/taShopkeeper/{id}",method= RequestMethod.GET)
//    @ApiOperation(value="详情", notes = "详情", httpMethod = "GET", response = ResponseBean.class)
//    public ResponseBean taShopkeeperGet(@ApiParam("对象ID") @PathVariable Integer id) throws Exception{
//        return ResponseBean.success(iTaShopkeeperService.getById(id));
//    }
}
