package com.yunzhi.niucai.service.impl;

import com.yunzhi.niucai.common.CommConstants;
import com.yunzhi.niucai.entity.TaCustomer;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaOrderPay;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaOrderPayMapper;
import com.yunzhi.niucai.service.ITaCustomerService;
import com.yunzhi.niucai.service.ITaOrderPayService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.niucai.service.ITaShopService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 订单支付  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-10-08
 */
@Service
public class TaOrderPayServiceImpl extends ServiceImpl<TaOrderPayMapper, TaOrderPay> implements ITaOrderPayService {

    @Autowired
    ITaShopService iTaShopService;

    @Autowired
    ITaCustomerService iTaCustomerService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public TaOrderPay createdBy(TaOrder taOrder, String payClient, String tradeType) throws Exception {
        TaOrderPay taOrderPay = new TaOrderPay();
        taOrderPay.setOrderId(taOrder.getOrderId());
        taOrderPay.setTradeNo(taOrder.getOrderId() + "-" + randomString(3));
        taOrderPay.setTradeType(tradeType);
        taOrderPay.setPayClient(payClient);
        taOrderPay.setPayFee(taOrder.getCharge());
//        taOrderPay.setPayReason();
        taOrderPay.setStatus(CommConstants.SRTATUS_UNPAYED);    // 未支付

        // tradeType 类似 b2c  c2b 等格式
        String[] tradeSplit = tradeType.split("2");
        String tradeFrom = tradeSplit[0];
        String tradeTo = tradeSplit[1];

        Map<String, String> payProfile = getPayTermProfile(tradeFrom, taOrder.getCustomerId());
        if (null != payProfile) {
            taOrderPay.setPayee(payProfile.get("id"));
            taOrderPay.setPayeeName(payProfile.get("name"));
        }

        if (null != taOrder.getShopId()) {
            Map<String, String> remitterProfile = getPayTermProfile(tradeTo, taOrder.getShopId());
            if (null != remitterProfile) {
                taOrderPay.setRemitter(remitterProfile.get("id"));
                taOrderPay.setRemitterName(remitterProfile.get("name"));
            }
        } else {
            // 订单中 shopId 为空, 说明是平台端
            // todo
        }

        save(taOrderPay);

        // 重新获取是因为有部分字段有默认值
        return getById(taOrderPay.getSerialNo());
    }

    private Map<String, String> getPayTermProfile(String tradeFrom, Integer customerId) throws Exception {
        // 如果是商户
        if ("b".equals(tradeFrom)) {
            TaShop taShop = iTaShopService.getById(customerId);
            if (null == taShop || StatusEnum.DELETED.getCode().equals(taShop.getStatus())) {
                throw new Exception(String.format("获取订单干系人失败: 商户ID [%s]", customerId));
            }

            return new HashMap<String, String>(){{
               put("id", taShop.getShopId().toString());
               put("name", taShop.getName());
            }};
        }

        if ("c".equals(tradeFrom)) {
            TaCustomer taCustomer = iTaCustomerService.getById(customerId);
            if (null == taCustomer || StatusEnum.DELETED.getCode().equals(taCustomer.getStatus())) {
                throw new Exception(String.format("获取订单干系人失败: 客户ID [%s]", customerId));
            }

            return new HashMap<String, String>(){{
                put("id", taCustomer.getCustomerId().toString());
                put("name", taCustomer.getName());
            }};
        }

        return null;
    }

    /**
     * 生成随机数
     * @param length
     * @return
     */
    public String randomString(int length) {
        String str = new BigInteger(130, new SecureRandom()).toString(32);
        return str.substring(0, length);
    }
}
