package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.yunzhi.niucai.common.CommConstants;
import com.yunzhi.niucai.common.DateUtils;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.TaBettingPlan;
import com.yunzhi.niucai.entity.TaCustomerBetting;
import com.yunzhi.niucai.entity.TaOrder;
import com.yunzhi.niucai.entity.TaShop;
import com.yunzhi.niucai.enums.OrderTypeEnum;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaOrderMapper;
import com.yunzhi.niucai.service.ITaBettingPlanService;
import com.yunzhi.niucai.service.ITaCustomerBettingService;
import com.yunzhi.niucai.service.ITaOrderService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.niucai.vo.DrawOrderParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 订单主表  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-09
 */
@Service
public class TaOrderServiceImpl extends ServiceImpl<TaOrderMapper, TaOrder> implements ITaOrderService {

    @Autowired
    ITaCustomerBettingService iTaCustomerBettingService;

    @Autowired
    ITaBettingPlanService iTaBettingPlanService;

    @Autowired
    TaOrderMapper taOrderMapper;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public TaOrder bettingOrder(TaBettingPlan taBettingPlan, TaCustomerBetting taCustomerBetting) throws Exception {
        TaOrder order = new TaOrder();
        order.setOrderId(nextOrderNo("BL", 16, true));
        order.setOrderType(OrderTypeEnum.BUY_LOTTERY.getCode());
        order.setAmount(taCustomerBetting.getBettingTimes());
        order.setCharge(taCustomerBetting.getTotalPrice());
        order.setShopId(taCustomerBetting.getShopId());
        order.setCustomerId(taCustomerBetting.getCustomerId());
        order.setBettingId(taCustomerBetting.getBettingId());
        order.setStatus(CommConstants.SRTATUS_UNPAYED); // 未支付

        if (!save(order)) {
            throw new Exception("生成购彩订单失败");
        }

        return order;
    }

    @Override
    public IPage<TaOrder> getShopOrders(IPage<TaOrder> pg, TaShop taShop, String lotteryId) throws Exception {
        if (taShop == null) {
            return null;
        }

        IPage<TaOrder> result = null;
        if (StringUtils.isEmpty(lotteryId)) {
            result = taOrderMapper.getShopOrders(pg, taShop.getShopId());
        } else {
            result = taOrderMapper.getShopOrdersWithLottery(pg, taShop.getShopId(), lotteryId);
        }

        // 带上投注信息
        if (null != result.getRecords() && result.getRecords().size() > 0) {
            for (TaOrder taOrder : result.getRecords()) {
                TaCustomerBetting customerBetting = iTaCustomerBettingService.getById(taOrder.getBettingId());
                if (null != customerBetting && !StatusEnum.DELETED.getCode().equals(customerBetting.getStatus())) {
                    TaBettingPlan bettingPlan = iTaBettingPlanService.getDetailOfPlanById(customerBetting.getPlanId());
                    customerBetting.setBettingPlan(bettingPlan);
                    taOrder.setCustomerBetting(customerBetting);
                }
            }
        }

        return result;
    }

    @Override
    public TaOrder getShopOrderDetail(TaShop taShop, String orderId) throws Exception {
        if (null == taShop) {
            return null;
        }

        TaOrder taOrder = taOrderMapper.getShopOrder(taShop.getShopId(), orderId);
        if (taOrder != null) {
            Map<String, Object> bettingDetail = iTaCustomerBettingService.getSummaryOf(taOrder.getBettingId());
            taOrder.setBettingDetail(bettingDetail);
        }

        return taOrder;
    }

    @Override
    public boolean drawOrder(TaOrder taOrder, DrawOrderParam drawOrderParam) throws Exception {
        TaCustomerBetting customerBetting = iTaCustomerBettingService.getById(taOrder.getBettingId());
        if (customerBetting == null) {
            throw new Exception("未找到有效的投注记录");
        }

        if (customerBetting.getIsDrawn()) {
            throw new Exception("已经出票, 无需操作");
        }

        customerBetting.setIsDrawn(true);
        customerBetting.setLotteryPicture(drawOrderParam.getLotteryPicture());

        return iTaCustomerBettingService.updateById(customerBetting);
    }

    @Override
    public TaOrder getByOutTradeNo(String outTradeNo) {
        return taOrderMapper.getByOutTradeNo(outTradeNo);
    }

    @Override
    public TaOrder getByBetting(TaCustomerBetting betting) throws Exception {
        QueryWrapper<TaOrder> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("betting_id", betting.getBettingId());
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());

        TaOrder taOrder = getOne(queryWrapper);

        return taOrder;
    }

    private String nextOrderNo(String prefix, int length, boolean padDate) throws Exception {
        List<String> parts = new ArrayList<>();
        parts.add(prefix);

        String dtStr = null;
        if (padDate) {
            dtStr = DateUtils.todayCompact();
            parts.add(dtStr);
        }

        int fillLength = length - prefix.length();
        if (padDate) {
            fillLength -= dtStr.length();
        }

        String nextNo = taOrderMapper.nextval(String.join("-", parts)).toString();

        String zeros = StringUtils.repeat("0", fillLength - nextNo.length());
        parts.add(zeros+nextNo);

        return String.join("-", parts);
    }
}
