package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yunzhi.niucai.common.ResponseBean;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaShopMapper;
import com.yunzhi.niucai.service.ITaShopLotteryService;
import com.yunzhi.niucai.service.ITaShopService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.niucai.service.ITaShopkeeperService;
import com.yunzhi.niucai.service.ITdCityService;
import com.yunzhi.niucai.vo.ShopKeeperSignUp;
import com.yunzhi.niucai.vo.ShopValidParam;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 店铺 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-08-28
 */
@Service
public class TaShopServiceImpl extends ServiceImpl<TaShopMapper, TaShop> implements ITaShopService {

    @Autowired
    ITaShopLotteryService iTaShopLotteryService;

    @Autowired
    ITaShopkeeperService iTaShopkeeperService;

    @Autowired
    ITdCityService iTdCityService;

    @Autowired
    TaShopMapper taShopMapper;

    @Override
    public Map<String, Object> getAvailableList(TaCustomer customer, String lotteryId, String location) throws Exception {
        // 获取推荐我的店铺
        TaShop ofMine = taShopMapper.getRecommendOf(customer.getCustomerId(), lotteryId);

        // 获取系统推荐列表
        List<TaShop> recommList = taShopMapper.getSysRecommendList(lotteryId);
        // 不包含推荐我的店铺
        if (null != ofMine && null !=recommList) {
            for(TaShop x : recommList) {
                if (x.getShopId().equals(ofMine.getShopId())) {
                    recommList.remove(x);
                    break;
                }
            }
        }

        // 获取最近距离的 10 家店铺
        // 不包含上面的结果
        List<Integer> notIn = new ArrayList<>();
        if (null != ofMine) {
            notIn.add(ofMine.getShopId());
        }
        if (null != recommList) {
            for (TaShop s : recommList) {
                notIn.add(s.getShopId());
            }
        }
        String[] loc = location.split(",");
        List<TaShop> shopList = taShopMapper.getNearestShops(lotteryId, loc[0], loc[1], notIn, 10);

        if (null == ofMine && null == recommList && null == shopList) {
            return null;
        }

        Map<String, Object> result = new HashMap<>();
        result.put("mine", ofMine);
        result.put("recommend", recommList);
        result.put("nearest", shopList);

        return result;
    }

    @Override
    public TaShop getByInviteCode(String inviteCode) throws Exception {
        QueryWrapper<TaShop> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("invite_code", inviteCode);
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());

        return getOne(queryWrapper);
    }

    @Override
    public TaShop getProfileBy(TaShopkeeper shopkeeper) throws Exception {
        QueryWrapper<TaShop> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("keeper_id", shopkeeper.getKeeperId());
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());

        TaShop taShop = getOne(queryWrapper);
        if (null == taShop) {
            return null;
        }

        // 获取彩种设置
        List<TaShopLottery> lotteryList = iTaShopLotteryService.getListBy(taShop);
        taShop.setLotteryList(lotteryList);
        return taShop;
    }

    @Override
    public boolean createBy(TaShopkeeper shopkeeper, ShopKeeperSignUp signUpParam) throws Exception {
        // 校验店主
        // 是否允许店主唯一, 业务暂时没有说明, 所以先不做校验

        TdCity province = iTdCityService.getByLevel(signUpParam.getProvince(), 1);
        if (null == province) {
            throw new Exception("未找到匹配的省份信息");
        }
        Integer provinceId = province.getAreaId();

        // 地级市
        TdCity city = iTdCityService.getByLevel(signUpParam.getCity(), 2);
        if (null == city) {
            city = iTdCityService.getByLevel(signUpParam.getCity(), 3);
            if (null == city) {
                throw new Exception("未找到匹配的城市信息");
            }
        }
        Integer cityId = city.getAreaId();

        if (!city.getParentid().equals(provinceId)) {
            throw new Exception("内部映射店铺地址出错");
        }

        if (StringUtils.isEmpty(signUpParam.getLng()) || StringUtils.isEmpty(signUpParam.getLat())) {
            throw new Exception("请设置店铺经纬度");
        }

        TaShop taShop = new TaShop();
        taShop.setProvinceId(provinceId.toString());
        taShop.setCityId(cityId.toString());
        taShop.setAddress(signUpParam.getAddress());
        taShop.setLng(signUpParam.getLng());
        taShop.setLat(signUpParam.getLat());

        if (null != shopkeeper) {
            taShop.setKeeperId(shopkeeper.getKeeperId());
        }

        // todo 账户信息

        return save(taShop);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ShopValidParam updateValidInfo(ShopValidParam shopValidParam, TaShop taShop, TaShopkeeper taShopkeeper) throws Exception {

        // 店铺
        taShop.setName(shopValidParam.getShop().getName());
        taShop.setBusinessLicense(shopValidParam.getShop().getBusinessLicense());
        taShop.setLat(shopValidParam.getShop().getLat());
        taShop.setLng(shopValidParam.getShop().getLng());
        taShop.setAddress(shopValidParam.getShop().getAddress());

        if (!StringUtils.isNumeric(shopValidParam.getShop().getProvinceId())) {
            TdCity province = iTdCityService.getByLevel(shopValidParam.getShop().getProvinceId(), 1);
            if (null != province) {
                taShop.setProvinceId(province.getAreaId().toString());
            }
        }

        if (!StringUtils.isNumeric(shopValidParam.getShop().getCityId())) {
            TdCity city = iTdCityService.getByLevel(shopValidParam.getShop().getCityId(), 2);
            if (null == city) {
                city = iTdCityService.getByLevel(shopValidParam.getShop().getCityId(), 3);
            }

            if (null != city) {
                taShop.setCityId(city.getAreaId().toString());
            }
        }

        if (!updateById(taShop)) {
            throw new Exception("更新店铺信息失败");
        }

        // 店主
        taShopkeeper.setName(shopValidParam.getShopkeeper().getName());
        taShopkeeper.setPhone(shopValidParam.getShopkeeper().getPhone());
        taShopkeeper.setPassword(null); // 不更新

        if (!StringUtils.isEmpty(shopValidParam.getShopkeeper().getFrontIdCard())) {
            taShopkeeper.setFrontIdCard(shopValidParam.getShopkeeper().getFrontIdCard());
            taShopkeeper.setBackIdCard(shopValidParam.getShopkeeper().getBackIdCard());
        }

        if (!iTaShopkeeperService.updateById(taShopkeeper)) {
            throw new Exception("更新店主信息失败");
        }

        shopValidParam.setShop(taShop);
        shopValidParam.setShopkeeper(taShopkeeper);
        return shopValidParam;
    }

    @Override
    public Map<String, Object> getSummeryByOrder(TaOrder taOrder) throws Exception {
        if (null == taOrder || null == taOrder.getShopId()) {
            return null;
        }

        return taShopMapper.getSummeryById(taOrder.getShopId());
    }
}
