package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.EncryptUtils;
import com.yunzhi.niucai.entity.TaShopkeeper;
import com.yunzhi.niucai.entity.TaShopkeeperThird;
import com.yunzhi.niucai.enums.StatusEnum;
import com.yunzhi.niucai.mapper.TaShopkeeperMapper;
import com.yunzhi.niucai.service.ITaShopService;
import com.yunzhi.niucai.service.ITaShopkeeperService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yunzhi.niucai.service.ITaShopkeeperThirdService;
import com.yunzhi.niucai.vo.ShopKeeperSignUp;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 店主 服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-28
 */
@Service
public class TaShopkeeperServiceImpl extends ServiceImpl<TaShopkeeperMapper, TaShopkeeper> implements ITaShopkeeperService {

    @Autowired
    ITaShopService iTaShopService;

    @Autowired
    ITaShopkeeperThirdService iTaShopkeeperThirdService;

    @Override
    public boolean signUp(ShopKeeperSignUp signUpParam) throws Exception {
        // 校验手机号唯一
        TaShopkeeper keeperBeChecked = getByPhone(signUpParam.getPhone());
        if (null != keeperBeChecked) {
            throw new Exception("当前手机号码已经注册过店铺");
        }

        // 生成店主信息
        TaShopkeeper shopkeeper = new TaShopkeeper();
        shopkeeper.setName(signUpParam.getName());
        shopkeeper.setPhone(signUpParam.getPhone());

        if (!save(shopkeeper)) {
            throw new Exception("设置店主信息出错, 请重试");
        }

        String newPassword = EncryptUtils.md5(signUpParam.getPassword(), shopkeeper.getKeeperId().toString());
        shopkeeper.setPassword(newPassword);
        if (!updateById(shopkeeper)) {
            throw new Exception("设置店主信息出错, 请重试");
        }

        return iTaShopService.createBy(shopkeeper, signUpParam);
    }

    @Override
    public boolean bindThirdPartId(TaShopkeeper taShopkeeper, String thirdPartId, String thirdPartType) throws Exception {
        TaShopkeeperThird taShopkeeperThird = iTaShopkeeperThirdService.getByKeeper(taShopkeeper, thirdPartType);
        if (null == taShopkeeperThird) {
            taShopkeeperThird = new TaShopkeeperThird();
            taShopkeeperThird.setThirdPartType(thirdPartType);
            taShopkeeperThird.setKeeperId(taShopkeeper.getKeeperId());
            taShopkeeperThird.setThirdPartId(thirdPartId);
            return iTaShopkeeperThirdService.save(taShopkeeperThird);
        } else {
            taShopkeeperThird.setThirdPartId(thirdPartId);
            return iTaShopkeeperThirdService.updateById(taShopkeeperThird);
        }
    }

    public TaShopkeeper getByPhone(String phone) {
        QueryWrapper<TaShopkeeper> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", phone);
        queryWrapper.gt("status", StatusEnum.DELETED.getCode());

        return getOne(queryWrapper);
    }
}
