package com.yunzhi.niucai.wxpay;

import com.alibaba.fastjson.JSONObject;
import com.github.binarywang.wxpay.bean.notify.WxPayOrderNotifyResult;
import com.github.binarywang.wxpay.bean.request.WxPayUnifiedOrderRequest;
import com.github.binarywang.wxpay.bean.result.WxPayUnifiedOrderResult;
import com.github.binarywang.wxpay.config.WxPayConfig;
import com.github.binarywang.wxpay.constant.WxPayConstants;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.yunzhi.niucai.common.StringUtils;
import com.yunzhi.niucai.wxpay.vo.WxPayParam;
import com.yunzhi.niucai.wxpay.vo.WapScene;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

@Slf4j
public class WxPayService {

    com.github.binarywang.wxpay.service.WxPayService wxPayService;

    WxPayConfig config;

    public WxPayService(WxPayConfig globalConfig, ISubMchResolve subMchResolve, Integer shopId) {
        config = new WxPayConfig();
        config.setAppId(globalConfig.getAppId());
        config.setMchId(globalConfig.getMchId());
        config.setSubMchId(subMchResolve.getSubMchId(shopId));
        // 暂时只支持 H5
        config.setTradeType(WxPayConstants.TradeType.MWEB);
        wxPayService.setConfig(config);
    }

    /**
     * 微信支付
     * @param request
     * @param payParam
     * @throws Exception
     */
    public WxPayUnifiedOrderResult pay(HttpServletRequest request, WxPayParam payParam) throws Exception {
        try {
            String ip = WxPayService.getRealIP(request);
            WxPayUnifiedOrderRequest orderRequest = new WxPayUnifiedOrderRequest();
            orderRequest.setBody(payParam.getBody());
            orderRequest.setOutTradeNo(payParam.getOutTradeNo());
            orderRequest.setTotalFee(payParam.getTotalFee());
            orderRequest.setSpbillCreateIp(ip);
            orderRequest.setTimeStart(payParam.getTimeStart());
            orderRequest.setTimeExpire(payParam.getTimeExpire());
            orderRequest.setNotifyUrl(payParam.getNotifyUrl());

            if (StringUtils.isEmpty(payParam.getSceneInfo())) {
                // 固定 H5
                // TODO
                WapScene scene = new WapScene();
                orderRequest.setSceneInfo(JSONObject.toJSONString(scene));
            }

            return wxPayService.unifiedOrder(orderRequest);
        } catch (Exception e) {
            log.error("微信支付失败！订单号：{},原因:{}", payParam.getOutTradeNo(), e.getMessage());
            e.printStackTrace();
            throw e;
        }
    }


    public WxPayOrderNotifyResult getOrderNotifyResult(String xmlData) throws WxPayException {
        return wxPayService.parseOrderNotifyResult(xmlData);
    }

    public static String getRealIP(HttpServletRequest request) {
        String ip = request.getHeader("X-Forwarded-For");
        if(!StringUtils.isEmpty(ip) && !"unKnown".equalsIgnoreCase(ip)){
            //多次反向代理后会有多个ip值，第一个ip才是真实ip
            int index = ip.indexOf(",");
            if(index != -1){
                return ip.substring(0,index);
            }else{
                return ip;
            }
        }
        ip = request.getHeader("X-Real-IP");
        if(!StringUtils.isEmpty(ip) && !"unKnown".equalsIgnoreCase(ip)){
            return ip;
        }
        return request.getRemoteAddr();
    }
}
